(function (angular, $, _) {
  "use strict";

  angular.module('crmKanban').component('crmSearchDisplayKanban', {
    bindings: {
      search: '<',
      display: '<',
      apiParams: '<',
      settings: '<',
      filters: '<',
      columnEntity: '@',
      includeReserved: '<',
      totalCount: '=?'
    },
    require: {
      afFieldset: '?^^afFieldset'
    },
    templateUrl: '~/crmKanban/crmSearchDisplayKanban.html',
    controller: function ($scope, $element, crmApi4, searchDisplayBaseTrait, searchDisplayTasksTrait) {
      var ts = $scope.ts = CRM.ts('org.civicrm.search_kit');

      // Mix in required traits
      angular.extend(this, _.cloneDeep(searchDisplayBaseTrait), _.cloneDeep(searchDisplayTasksTrait));

      this.$onInit = () => {
        this.initializeDisplay($scope, $element);

        this.initKanbanColumns();

        this.onPostRun.push(this.resultRowsToCards);
      };

      this.initKanbanColumns = () => {
        this.kanbanColumns = [];

        crmApi4(this.columnEntity, 'get', {
          'select': ['id', 'label', 'weight', 'is_reserved'],
        }).then((results) => {
          results.forEach((result) => {
            if (this.includeReserved || !result.is_reserved) {
              this.kanbanColumns.push({
                key: result.id,
                label: result.label,
                cards: [],
                count: 0,
                weight: parseInt(result.weight)
              });
            }
          });
          this.kanbanColumns.sort((a, b) => a.weight < b.weight ? -1 : 1);
        });
      };

      this.resultRowsToCards = () => {
        this.loading = true;

        const byKey = {};

        if (this.results.length) {
          const cardLinkColumn = this.settings.cardLinkColumn;
          const cardLinkColumnIndex = cardLinkColumn ? this.settings.columns.findIndex((col) => col.key === cardLinkColumn) : -1;

          for (const row of this.results) {
            const keyColumnValue = row.columns[0].val;

            // ensure target column exists
            byKey[keyColumnValue] = byKey[keyColumnValue] ?? [];

            if (cardLinkColumnIndex >= 0) {
              row.href = row.columns[cardLinkColumnIndex].val;
              // blank val so it isn't displayed
              row.columns[cardLinkColumnIndex].val = null;
            }

            byKey[keyColumnValue].push(row);
          }
        }

        for (const column of this.kanbanColumns) {
          column.cards = byKey[column.key] ?? [];
          column.count = column.cards.length;
        }
        this.loading = false;
      };
    }
  });
})(angular, CRM.$, CRM._);
